--[[
	To Do:
		-- fix draenor and dungeons! Might be a blizzard issue.
		-- Eversong Woods does not return coords for Azeroth, returns for outland. needs adjustments.
--]]


Questra = {}

local HBD = LibStub:GetLibrary("HereBeDragons-2.0", true)
local CTL = assert(ChatThrottleLib, "Quest_Compass requires ChatThrottleLib.")

local Quest_Compass = CreateFrame("Frame", ni, UIParent)
Quest_Compass.position = {x=0, y=0}

Quest_Compass:RegisterEvent("ADDON_LOADED")
Quest_Compass:RegisterEvent("ZONE_CHANGED_NEW_AREA")
Quest_Compass:RegisterEvent("ZONE_CHANGED_INDOORS")
Quest_Compass:RegisterEvent("ZONE_CHANGED")

Quest_Compass:SetScript("OnEvent", function(self, event, arg1)
	if (event == "ADDON_LOADED") and arg1 == "Quest_Compass" then
		self:UpdateSettings()
		self:Create()
	else
		self:Update()
	end
end)

function Quest_Compass:UpdateSettings()
	local sets
	if not Quest_CompassDB then
		sets = {version = 0, X = 90, Y = 0}
	else
		sets = Quest_CompassDB or {version = 1, X = 90, Y = 0}
		sets.version = sets.version + 1
	end

	Quest_CompassDB = Quest_CompassDB or sets
	self.sets = Quest_CompassDB
	return self.sets
end

local function OverFrame(frame)
	local overFrame
	local _, f = DropDownList1:GetPoint()
	if DropDownList1:IsShown() and MouseIsOver(DropDownList1) and frame.dropDown and frame.dropDown.buttonOwner and f == frame.dropDown.buttonOwner then
			return true
	elseif MouseIsOver(frame) then
		return true
	end
end

function Quest_Compass:GetCursorPosition()
	local currentX, currentY = GetCursorPosition()
	local effectiveScale = UIParent:GetEffectiveScale()
	return currentX / effectiveScale, currentY / effectiveScale
end

function Quest_Compass:Create()
	self:SetPoint("Center", UIParent)
	self:SetSize(100, 100)

	self.coords = self.coords or CreateFrame("EditBox", "Quest_CompassEditBox", UIParent, "InputBoxScriptTemplate")
	self.coords:SetFontObject("GameFontHighlight")
	self.coords:SetClampedToScreen(true)
	self.coords:SetJustifyH("CENTER")
	self.coords:SetAutoFocus(false)
	self.coords:SetPoint("Center", 0, 0)
	self.coords:SetMovable(true)
	self.coords:SetSize(98, 20)

	self.coords:SetHighlightColor(.5,1,.5)

	self.coords.mover = self.coords.mover or CreateFrame("Frame", "Quest_CompassLocationDisplay", self)
	self.coords.mover:SetAllPoints(self.coords)
	self.coords.mover:EnableMouse(true)
	self.coords.mover:Hide()

	self.coords.mover.blue = self.coords.mover:CreateTexture(nil, 'OVERLAY')
	self.coords.mover.blue:SetAllPoints(self.coords.mover)
	self.coords.mover.blue:SetColorTexture(0,0,1, .25)

	self.compassManager = self.compassManager or CreateFrame("Frame", nil, self.coords)
	self.compassManager:SetAllPoints(self.coords)

	self.compassClick = self.compassClick or CreateFrame("Frame", nil, self.coords)
	self.compassClick:SetSize(100, 100)
	self.compassClick:SetPoint("Center", 0, 50)
	self.compassClick:SetScript("OnMouseUp", function(_,btn)
		self:IgnoreQuest(btn)
	end)

	self.compass = self.compass or self.compassManager:CreateTexture(nil, "ARTWORK")

	self.pin = self.pin or CreateFrame("Button", nil, self.coords)
	self.pin:SetSize(20, 20)
	self.pin:SetPoint("Center", self.compass)
	self.pin:SetScale(3)
		local pin = self.pin
		if not pin.Set then
			pin.Set = true

			pin.Background = pin:CreateTexture(nil, "BACKGROUND")
			pin.Background:SetDrawLayer("BACKGROUND", -1)
			pin.Background:SetPoint("Center")

			pin.PushedBackground = pin:CreateTexture(nil, "BACKGROUND")
			pin.PushedBackground:SetDrawLayer("BACKGROUND", -1)
			pin.PushedBackground:SetPoint("Center")
			pin.PushedBackground:Hide()

			pin.Texture = pin:CreateTexture(nil, "ARTWORK")
			pin.Texture:SetPoint("Center")

			pin.Highlight = pin:CreateTexture(nil, "HIGHLIGHT")
			pin.Highlight:SetPoint("Center")
			pin.Highlight:SetBlendMode("ADD")

			pin.ring = pin:CreateTexture(nil, "ARTWORK")
			pin.ring:SetPoint("Center")
			pin.ring:SetSize(35, 35)
			pin.ring:SetTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\UI-MINIMAP-BORDER")

			pin:SetScript("OnEnter", function() Quest_Compass:TaskPOI_OnEnter(pin) end)
			pin:SetScript("OnLeave", function() Quest_Compass:TaskPOI_OnLeave(pin) end)

			pin:SetScript("OnMouseUp", function(_,btn)
				self:IgnoreQuest(btn)
			end)
		end

	self.compass:SetAllPoints(self.compassClick)
	self.compassClick:EnableMouse(false)

	self.icon = self.icon or self.compassManager:CreateTexture(nil, "BACKGROUND")

	self.icon:SetSize(55,55)
	self.icon:SetPoint("Center", self.compass)

	self.textBG = self.textBG or self.coords:CreateTexture()
	self.textBG:SetTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\UI-MINIMAP-BORDER-TextBorder")
	self.textBG:SetSize(115,115)
	self.textBG:SetPoint("Center", 0, 40)
	
	self.arrow = self.arrow or self.compassManager:CreateTexture(nil, "OVERLAY")
	self.arrow:SetTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\WorldMapArrow")
	self.arrow:SetPoint("Center", self.compass)
	self.arrow:SetSize(100, 100)

	self.distText = self.distText or self.compassManager:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	self.distText:SetPoint("Bottom", self.compass, "Top", 0, -12)

	self:SetScript("OnUpdate", self.OnUpdate)

	self.coords:SetScript("OnEditFocusGained", function() self.coords.hasFocus = true self.coords:SetText("")  end)
	self.coords:SetScript("OnEditFocusLost", function() self.coords.hasFocus = nil self.coords:HighlightText(0,0) end)
	self.coords:SetScript("OnEscapePressed", function() self.coords:ClearFocus() end)
	self.coords:SetScript("OnEnterPressed",	function() self:OnEnterPressed() end)

	self.coords.mover:SetScript("OnMouseDown",	function(_,btn) self:OnMouseDown(btn) end)
	self.coords.mover:SetScript("OnMouseUp",	function() self:OnMouseUp() end)
	self.coords.mover:SetScript("OnEnter",	function() self:OnEnter() end)
	self.coords.mover:SetScript("OnMouseWheel",	function(_, delta) self:OnMouseWheel(delta) end)

	WorldMapFrame:SetClampedToScreen(true)
	
	
	self:Layout()
end

Quest_Compass.defaults = {
		point = "Center",
		x = 0,
		y = -25,
		scale = 1,
		layer = 4,
		corpse = true,
		quests = true,
		worldQuests = true,
		nearest_worldQuests = true,
		distance = 1,
		showDist = true,
		corpse = true,
		quests = true,
		worldQuests = true,
		nearest_worldQuests = true,
		ignored = {},
		ignoreQuestTypes = {},
	}

local options = {
		{
			name = "Map Line",
			key = "line",
			style = "Toggle",
		},
		{
			name = "Flip",
			key = "flip",
			style = "Toggle",
		},
		{
			name = "show Distance",
			key = "showDist",
			style = "Toggle",
		},
		{
			name = "Corpse Location",
			key = "corpse",
			style = "Toggle",
		},
		{
			name = "Quest Location",
			key = "quests",
			style = "Toggle",
		},
		{
			name = "World Quests",
			key = "worldQuests",
			style = "Toggle",
		},
		{
			name = "Nearest World Quests",
			key = "nearest_worldQuests",
			style = "Toggle",
		},
		{
			name = "Distance Sensitivity",
			key = "distance",
			style = "Slider",
			low = 0,
			high = 100,
			displayAdjust = 1,
		},

		-- {
			-- name = "Frame layer",
			-- key = "layer",
			-- style = "Slider",
			-- low = 1,
			-- high = 8,
			-- cap = true,
		-- },
		{
			name = "Scale",
			key = "scale",
			style = "Slider",
			low = 25,
			high = 250,
			displayAdjust = 100,
			display = true,
		},
		
		{
			name = "Ignore Quest Types",
			key = "ignoreQuestTypes",
			style = "Menu",
			options = {
				"Battle Pet",
				"Dungeon",
				"PvP",
				"Profession",
				"Raid",
				"Invasion"
			},
			display = true,
		},
		
	}
	
	
Quest_Compass.options = options

function Quest_Compass:GetDefaults()
	return self.defaults
end

do --Settings
	function Quest_Compass:CreateDropDownMenu()
		if not self.dropdown then
			local dropdown = CreateFrame("Frame", "Quest_CompassOptionsMenu", self.coords, "UIDropDownMenuTemplate")
			self.dropdown = dropdown
			UIDropDownMenu_Initialize( dropdown, function(_, level, ...) return self:InitializeMenu(level, ...) end, "MENU")
		end
		return self.dropdown
	end

	function Quest_Compass:InitializeMenu(level, ...)
		if level == 1 then
			for i, inf in pairs(self:GetMenuOptions(level)) do
				if inf.dispCondition() then
					local info = UIDropDownMenu_CreateInfo()
					for j, b in pairs(inf) do
						info[j] = b
					end
					UIDropDownMenu_AddButton(info)
				end
			end
		else
			local inf = self:GetMenuOptions(level, UIDROPDOWNMENU_MENU_VALUE)
			for i, inf in pairs(inf or {}) do
				local info = UIDropDownMenu_CreateInfo()
				for j, b in pairs(inf) do
					info[j] = b
				end
				UIDropDownMenu_AddButton(info, 2)
			end
		end
	end

	function Quest_Compass:GetMenuOptions(level, menuValue)
		if level == 1 then
			if not self.MenuOne then
				self.MenuOne = {}
				for key, info in pairs(options) do
					local inf = {}
					inf.text = info.name
					if info.style == "Toggle" then
						if not info.func then
							inf.func = function() return self:ToggleSetting(info.key) end
						else
							inf.func = function() return info.func(self) end
						end
						if not info.checked then
							inf.checked = function() return self:GetSetting(info.key) end
						else
							inf.checked = function() return info.checked(self) end
						end
					elseif info.style == "Slider" then
						inf.displayMode = "MENU"
						inf.hasArrow = 1
						inf.value = info.key
						inf.notCheckable = true
					elseif info.style == "Menu" then
						inf.displayMode = "MENU"
						inf.hasArrow = 1
						inf.value = info.key
						inf.notCheckable = true
					end
					inf.keepShownOnClick = true
					inf.dispCondition = function()
						if info.display then
						--	if not (placeHolder) then
								return true
						--	end
						else
							return true
						end
					end,
					tinsert(self.MenuOne, inf)
				end
			end

			return self.MenuOne
		end
		if (level == 2) then
			local menu = {}
			if self:GetOptionInfo(menuValue).style == "Slider" then
				do
					
					local keys
					if self:GetOptionInfo(menuValue).cap then
						keys = {-1, "x", 1}
					else
						keys = {-25,-10,-5, -1, -.5, -.1, "x", .1, .5, 1,5,10,25}
					end

					for i, number in pairs(keys) do
						local inf = {}
						if number == "x" then
							local adjust = self:GetAdjust(menuValue) or 1
							inf.text = self:GetSetting(menuValue) * adjust
							inf.func = function()
								if IsShiftKeyDown() then
									self:ResetSetting(menuValue)

									_G["DropDownList2Button"..i]:SetText(self:GetSetting(menuValue) * adjust)
								end
							end
						else
							if number > 0 then
								inf.text = "+"..number
							else
								inf.text = number
							end
							inf.func = function()
								local adjust = self:GetAdjust(menuValue) or 1
								local high, low = self:GetRange(menuValue)

								local set = self:GetSetting(menuValue)

								local value = (set * adjust) + number

								local text = 0

								if value > high or value < low then
									if value > high then
										self:UpdateSetting(menuValue, high/adjust)
										text = high
									elseif value < low then
										self:UpdateSetting(menuValue, low/adjust)
										text = low
									end
								else

									self:AdjustSetting(menuValue, number/adjust)
									text = value
								end

								if _G["DropDownList2Button7"]:IsShown() then
									_G["DropDownList2Button7"]:SetText(text)
								else
									_G["DropDownList2Button2"]:SetText(text)
								end
								self:Layout()
							end
						end

						inf.notCheckable = true
						inf.keepShownOnClick = true
						tinsert(menu, inf)
					end
				end
			elseif self:GetOptionInfo(menuValue).style == "Menu" then
				
				local opts = self:GetOptionInfo(menuValue).options
				
				
				for _, b in pairs(opts) do
					local inf = {}
				
					inf.text = b
					inf.func = function() return self:ToggleDeepSetting(menuValue, b) end

					inf.checked = function() return self:GetDeepSetting(menuValue, b) end

					inf.keepShownOnClick = true
					tinsert(menu, inf)
				end
	
				
				
				
			end
			return menu
		end
	end

	function Quest_Compass:ToggleSetting(setting)
		self.sets[setting] = not self.sets[setting]
		if self.sets[setting] == false then
			self.sets[setting] = nil
		end
		self:Layout()
	end

	function Quest_Compass:ToggleDeepSetting(...)
		local inf = {...}
		if #inf > 1 then
		
			local index = 2

			local key = inf[1] 

			self.sets[key] = self.sets[key] or {}
			local set = self.sets[key]

			while index < #inf do
				key = inf[index]
			
				if inf[index+1] then
					set[key] = set[key] or {}				
				end
				set = set[key]
				index = index + 1
			end
			
	
			set[inf[#inf]] = not set[inf[#inf]]
			if set[inf[#inf]] == false then
				set[inf[#inf]] = nil
			end
			self:Layout()
		else
			return self:ToggleSetting(...)
		end
	end

	function Quest_Compass:UpdateSetting(setting, value)
		if self.sets[setting] then
			self.sets[setting] = value
		end
		self:Layout()
	end

	function Quest_Compass:GetAdjust(key)
		for i, info in pairs(options) do
			if info.key == key then
				return info.displayAdjust or 1
			end
		end
	end

	function Quest_Compass:GetRange(key)
		for i, info in pairs(options) do
			if info.key == key then
				return info.high, info.low
			end
		end
	end

	function Quest_Compass:GetOptionInfo(key)
		for i,b in pairs(options) do
			if b.key == key then
				return b
			end
		end
	end

	function Quest_Compass:ResetSetting(setting)
		self.sets[setting] = self.defaults[setting]
		self:Layout()
	end

	function Quest_Compass:AdjustSetting(setting, value)
		if (not self.sets[setting]) and self.defaults[setting] and (type(self.defaults[setting]) == "number") then
			self.sets[setting] = self.defaults[setting]
		end
		if self.sets[setting] then
			self.sets[setting] = self.sets[setting] + value
		end
		self:Layout()
	end


	function Quest_Compass:GetSetting(setting)
		if not self.sets then
			return self.defaults[setting]
		else
			if (not self.sets[setting]) and self.defaults[setting] and (type(self.defaults[setting]) == "number") then
				self.sets[setting] = self.defaults[setting]
			end
			return self.sets[setting]
		end
	end
	
	
	function Quest_Compass:GetDeepSetting(...)
		local inf = {...}
		if #inf > 1 then
		
			local index = 2

			local key = inf[1] 

			self.sets[key] = self.sets[key] or {}
			local set = self.sets[key]

			while index <= #inf do
				key = inf[index]
			
				if inf[index+1] then
					set[key] = set[key] or {}				
				end
				set = set[key]
				index = index + 1
			end
			return set
		else
			return self:GetSetting(...)
		end
	end
end

do
	local justify = {Left = 1, Right = -1}

	local function GetCoordButton(parent, justifyH, offset)
		local b = parent:CreateTexture(nil, "ARTWORK")
		b:SetTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\coordBG")

		local text = parent:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
		text:SetJustifyH(justifyH)

		local tX, bX = (15 + (offset or 0)) * justify[justifyH], 15 * justify[justifyH]
		if justifyH == "Right" then
			b:SetTexCoord(1, 0, 0, 1)
			tX, bX = bX, tX
		end

		text:SetPoint("TopLeft", b, tX, 0)
		text:SetPoint("BottomRight", b, bX, 0)

		return text, b
	end

	function Quest_Compass:RunOnce()
			local map = self:GetMap()
		if (not self.loaded) and map then
			self.loaded = true
			self:Layout()
			if map then
				self.mapCoordshandle = self.mapCoordshandle or CreateFrame("Frame", nil, map, "Quest_CompassNavBarTemplate")
				self.mapCoordshandle:SetFrameStrata("TOOLTIP")
				self.mapCoords = self.mapCoords or self.mapCoordshandle:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
				self.mapCoordshandle:SetScript("OnShow",nil)

				self.mapCoordshandle:SetPoint("Top", WorldMapFrame, "Bottom", 0, 0)
				self.mapCoordshandle:SetPoint("Left", WorldMapFrame, "Left", 5, 0)
				self.mapCoordshandle:SetPoint("Right", WorldMapFrame.NavBar, "Right", 2, 0)
				self.mapCoordshandle:SetHeight(40)
				self.mapCoordshandle:EnableMouse(true)
				self.mapCoordshandle:SetFrameStrata("HIGH")
				WorldMapFrame:HookScript("OnShow", function()
					QuestScrollFrame:SetPoint("Bottom", self.mapCoordshandle)
					QuestScrollFrame.Background:SetPoint("Bottom", 0, 0)
					QuestMapFrame.VerticalSeparator:SetPoint("Bottom", self.mapCoordshandle, 0, 0)
					WorldMapFrame.BorderFrame:SetPoint("Bottom", self.mapCoordshandle, 0 ,-8)
				end)

				local cpText, cpBG = GetCoordButton(self.mapCoordshandle, "Left", 10)
				self.playerCoordText = cpText

				local pText, pBG = GetCoordButton(self.mapCoordshandle, "Left")
				pText:SetText("Player")
				pBG:SetPoint("TopLeft" , self.mapCoordshandle, 3, -5)
				pBG:SetPoint("BottomRight" , self.mapCoordshandle, "BottomLeft", 100, 3)

				cpBG:SetPoint("TopLeft" , pBG, "TopRight", -20, 0)
				cpBG:SetPoint("BottomRight" , pBG, "BottomRight", 150, 0)

				local ccText, ccBG = GetCoordButton(self.mapCoordshandle, "Right", 10)
				self.cursorCoordText = ccText

				local cText, cBG = GetCoordButton(self.mapCoordshandle, "Right")
				cText:SetText("Cursor")
				cBG:SetPoint("TopRight" , self.mapCoordshandle, -3, -5)
				cBG:SetPoint("BottomLeft" , self.mapCoordshandle, "BottomRight", -100, 3)

				ccBG:SetPoint("TopRight" , cBG, "TopLeft", 20, 0)
				ccBG:SetPoint("BottomLeft" , cBG, "BottomLeft", -150, 0)

				self.playerCoordBG = self.mapCoordshandle:CreateTexture(nil, "ARTWORK")
				self.playerCoordBG:SetTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\coordBG")

				self.lineContainer = CreateFrame("ScrollFrame", nil, WorldMapFrame)
				self.lineManager = CreateFrame("Frame", nil, self.lineContainer)
				self.lineContainer:SetAllPoints(map)
				self.lineContainer:SetScrollChild(self.lineManager)
				self.lineManager:SetAllPoints(self.lineContainer)
				self.lineManager:EnableMouse(false)

				self.playerBox = CreateFrame("Frame", nil, UIParent)
				self.wayBox = CreateFrame("Frame", nil, UIParent)

				self.lineManager:SetScript("OnMouseDown", function(lineMan)
					self:ApplyWayPoint()
				end)
				self.line = self.line or self.lineManager:CreateLine(nil, "ARTWORK")
				self.line:SetThickness(9)
				self.line:SetColorTexture(1,1,1,1)
			end
		end
	end

	function Quest_Compass:ApplyWayPoint()
		local canvas = self:GetCanvas()
		local cursorX, cursorY = 0, 0
		if self:GetMap():IsShown() and canvas and MouseIsOver(canvas) then
			local curX, curY = GetCursorPosition()
			local scale = canvas:GetEffectiveScale()
			cursorX = ((curX / scale) - canvas:GetLeft()) / canvas:GetWidth()
			cursorY = (canvas:GetTop() - (curY / scale)) / canvas:GetHeight()
		end
		self:SetWayPoint(cursorX, cursorY)
	end

	function Quest_Compass:SetWayPoint(x, y, mapID)
		self.waypointX = x
		self.waypointY = y
		self.waypointID = mapID or self:GetViewedMapID()
	end
end

--[[Script Functions--]]
do
	function Quest_Compass:OnUpdate()
		self:Update()
		if not (MouseIsOver(self.coords) and IsShiftKeyDown()) and self.coords.mover:IsShown() then
			self.coords.mover:Hide()
		end
	end

	function Quest_Compass:OnEnterPressed()
		local t = string.gsub(self.coords:GetText(), ",", "")
		local x, y = string.split(" ", t)
		x, y = tonumber(x)/100,tonumber(y)/100
		if x and y then
			self:SetWayPoint(x, y, self:GetViewedMapID())
			self.coords:ClearFocus()
		end
	end

	function Quest_Compass:OnEnter()
		if IsShiftKeyDown() then
			self.coords.mover:Show()
		else
			self.coords.mover:Hide()
		end
	end

	local GetRelPos = function(self)
		local width, height = GetScreenWidth()/self:GetScale(), GetScreenHeight()/self:GetScale()
		local x, y = self:GetCenter()
		local xOffset, yOffset
		local Hori = (x > width/2) and 'RIGHT' or 'LEFT'
		if Hori == 'RIGHT' then
			xOffset = self:GetRight() - width
		else
			xOffset = self:GetLeft()
		end
		local Vert = (y > height/2) and 'TOP' or 'BOTTOM'
		if Vert == 'TOP' then
			yOffset = self:GetTop() - height
		else
			yOffset = self:GetBottom()
		end
		return Vert, Hori, xOffset, yOffset
	end

	local function SelectProperSide(self)
		local width = GetScreenWidth()

		local Vert, Hori, xOffset, yOffset = GetRelPos(self)
		local vert

		if Vert == "TOP" then
			vert = "BOTTOM"
		elseif Vert == "BOTTOM" then
			vert = "TOP"
		end

		return Vert..Hori, vert..Hori
	end
	Quest_Compass.SelectProperSide = SelectProperSide

	function Quest_Compass:OnMouseDown(btn)
		if btn == "RightButton" then
			ToggleDropDownMenu(1, 1, self:CreateDropDownMenu(), self.coords:GetName(), 0, -5)

			DropDownList1:ClearAllPoints()
			local p1, p2 = SelectProperSide(self.coords)
			DropDownList1:SetPoint(p1, self.coords, p2)
			return
		end

		self.mouseIsDown = true
		self.coords:StartMoving()
		self.coords:ClearFocus()
	end

	function Quest_Compass:OnMouseUp()
		local coords = self.coords
		coords:StopMovingOrSizing()
		coords:ClearFocus()
		self:SavePosition()
		self.mouseIsDown = nil
	end

	function Quest_Compass:OnMouseWheel(delta)
		self:Rescale(self:GetSets("scale", 1) + (delta/100))
	end
end

--[[Frame Position Functions--]]
do
	function Quest_Compass:SavePosition()
		local scale = self.coords:GetEffectiveScale()
		local x, y = self.coords:GetCenter()
		local pX, pY = UIParent:GetCenter()
		

		local w, h = UIParent:GetSize()
		local  uScale = UIParent:GetEffectiveScale()
		
		local uiX, uiY = pX * uScale, pY * uScale

		x, y = x*scale, y*scale

		local snapDist = 20
		
		if x >= (uiX - snapDist) and x <= (uiX + snapDist) then
			x = uiX
		end
		if y >= (uiY - snapDist) and y <= (uiY + snapDist) then
			y = uiY
		end
		x, y = x - uiX, y - uiY
		self.sets.X, self.sets.Y = x, y
		self:Reposition()
	end

	function Quest_Compass:Reposition()
		local x, y = self:GetSets().X, self:GetSets().Y
		self.coords:ClearAllPoints()
		local scale = self.coords:GetEffectiveScale()

		self.coords:SetPoint("Center", UIParent, "Center", x/scale, y/scale)
	end

	function Quest_Compass:Rescale(value)
		if not self.coords then return end
		if value then
			Quest_CompassDB.scale = value
		end
		local scale = self:GetSets("scale", 1)
		
		
		self.coords:SetScale(scale)
		if value then
			self:Reposition()
		end
	end

	function Quest_Compass:Layout()
		self:Rescale()
		self:Reposition()
		self.distText:ClearAllPoints()
		if	self.sets.flip then
			self.compassClick:SetPoint("Center", 0, -40)
			self.distText:SetPoint("Top", self.compass, "Bottom", 0, 17)
		else
			self.compassClick:SetPoint("Center", 0, 40)
			self.distText:SetPoint("Bottom", self.compass, "Top", 0, -17)
		end
	end

end

--[[ Display Updates--]]
do
	local mapInfo
	local ParentInfo

	local parentPaths = {}

	local rareTargetInfo = {}


	treasureTimer = math.huge
	local treasure = {
		[3365] = "Mechanized Chest"
	
		--add more!
	}

	local treasureX, treasureY, treasureMapID

	local trackedTreasure
	local function TrackTreasure(x, y, mapID)
		local name, text, texture, startTimeMS, endTimeMS, isTradeSkill, castID, notInterruptible, spellId = UnitCastingInfo("player")
		if treasure[spellId] then
			treasureTimer = GetTime()
			trackedTreasure = spellId
			treasureX, treasureY, treasureMapID = x, y, mapID
		end
		
		if treasureTimer then
			local elapsed = treasureTimer - GetTime()
			
			if (elapsed < -10) or treasureShared then
				treasureTimer = nil
				trackedTreasure = nil
				treasureX, treasureY, treasureMapID = nil, nil, nil
				treasureShared = nil
			end
		end
	end




	function Quest_Compass:Update()
		
		self:RunOnce()
		self:ShowMover()

		local viewedMapID = self:GetViewedMapID()
		local playerMapID = self:GetPlayerMapID()

		local cursorX, cursorY = 0, 0
		do --cursor Location
			local canvas = self:GetCanvas()
			if self:GetMap():IsShown() and canvas and MouseIsOver(canvas) then
				local curX, curY = GetCursorPosition()
				local scale = canvas:GetEffectiveScale()
				cursorX = ((curX / scale) - canvas:GetLeft()) / canvas:GetWidth()
				cursorY = (canvas:GetTop() - (curY / scale)) / canvas:GetHeight()
			end
		end

		local playerX, playerY = 0, 0

		do --player location
			if playerMapID then
				local px, py, pID = HBD:GetPlayerZonePosition(true)
				if not px then
					--player position doesn't seem to work on draenor or in dungeons
					playerX, playerY = 0, 0

					if ParentInfo and ParentInfo[playerMapID] ~= 572 then --not on draenor
						--placeholder for now, want to find a way to get position while in a dungeon
					end

				else
					playerX, playerY, playerMapID = px, py, pID
				end
			end
		end

		local waypointX, waypointY, waypointID = 0, 0

		do --waypoint location
			--[[
				~Priorities~
					-corpse
					-rare target
					-user Selected
					-standard quest
					-world quest
			--]]

			if (playerX ~=0) and (playerY~= 0) then

				if UnitIsGhost("player") == false then
					self.ignoreDeath = nil
				end
				
				
				local rare
				--rare target announcement.
				do
					if UnitExists("target") then
						local status = UnitClassification("target")
						--the unit's classification: "worldboss", "rareelite", "elite", "rare", "normal", "trivial", or "minus"						
						local alive = not UnitIsDead("target")
						
						local _, _, _, _, _, guid = strsplit("-", UnitGUID("target"))
						
						guid = tonumber(guid)
						if (status == "rare" or
							status == "elite" or
							status == "rareelite") and 
							alive and self.Rares[guid] then
							
							table.wipe(rareTargetInfo)
							
							rareTargetInfo.name =  UnitName("target")
							rareTargetInfo.health = math.floor((UnitHealth("target")/UnitHealthMax("target"))*100)
							rareTargetInfo.combat = UnitAffectingCombat("target")
							rareTargetInfo.x = playerX
							rareTargetInfo.y = playerY
							rare = true
							self.waypointType = "rare"
							waypointX, waypointY, waypointID = playerX, playerY, playerMapID
						end
					
					end
				end
				
				
				if not rare then
					if (self.sets.corpse == true) and UnitIsGhost("player") == true and self.ignoreDeath ~= true then
						local position = C_DeathInfo.GetCorpseMapPosition(playerMapID)
						if position then
							self.waypointType = "corpse"
							waypointX, waypointY, waypointID = position.x, position.y, playerMapID
						end
					elseif self.waypointX then
						self.waypointType = "userSelected"
						waypointX, waypointY, waypointID = self.waypointX, self.waypointY, self.waypointID
					else
						self.waypointType = "quest"
						waypointX, waypointY, waypointID = self:GetQuestLocation()
					end
				end

			else
				waypointX, waypointY = 0, 0
			end
			
			
			
			
				--treasure!
				do
					if trackedTreasure then
						waypointX, waypointY, waypointID = treasureX, treasureY, treasureMapID
						self.waypointType = "treasure"
					end
				end
				
			
		end

		self:UpdateWayPoint(playerX, playerY, waypointX, waypointY, playerMapID, waypointID)

		if playerX and playerY then
			self:UpdateCoordText(playerX*100, playerY*100, cursorX*100, cursorY*100 )
		end
	end

	function Quest_Compass:ShowMover()
		if MouseIsOver(self.coords) then
			if IsShiftKeyDown() then
				self.coords.mover:Show()
				self.coords:EnableMouse(false)
			else
				if self.mouseIsDown then
					self:OnMouseUp()
				end
				self.coords.mover:Hide()
				self.coords:EnableMouse(true)
			end
		end

		if MouseIsOver(self.lineContainer) and IsShiftKeyDown() then
			self.lineManager:EnableMouse(true)
		else
			self.lineManager:EnableMouse(false)
		end
	end

	function Quest_Compass:UpdateCoordText(playerX, playerY, cursorX, cursorY)
		if not self.coords then
			return
		end

		local playerString, cursorString = "", ""
		if playerX ~= 0 and playerY ~= 0 then
			playerString = format("%.2f, %.2f", playerX or 0, playerY or 0)
		end
		if cursorX ~= 0 and cursorY ~= 0 then
			cursorString = format("%.2f, %.2f", cursorX or 0, cursorY or 0)
		end

		if self.playerCoordText then
			if not self.coords.hasFocus  then
				self.coords:SetText(playerString)
			end
			self.playerCoordText:SetText(playerString)
			self.cursorCoordText:SetText(cursorString)
		end
	end
	local atlasList
	atlasList = {
		[LE_QUEST_TAG_TYPE_PVP] = function() return "worldquest-icon-pvp-ffa", true end,

		[LE_QUEST_TAG_TYPE_PET_BATTLE] = function() return "worldquest-icon-petbattle", true end,

		[LE_QUEST_TAG_TYPE_PROFESSION] = function(tradeskillLineID) return WORLD_QUEST_ICONS_BY_PROFESSION[tradeskillLineID], true end,

		[LE_QUEST_TAG_TYPE_DUNGEON] = function() return "worldquest-icon-dungeon", true end,

		[LE_QUEST_TAG_TYPE_RAID] = function() return "worldquest-icon-raid", true end,

		[LE_QUEST_TAG_TYPE_INVASION] = function() return "worldquest-icon-burninglegion", true end,
		
		[LE_QUEST_TAG_TYPE_FACTION_ASSAULT] = function()
			local faction = UnitFactionGroup("player") 
			if faction == "Horde" then
				return "QuestPortraitIcon-Horde-small", true, 20, 20
			elseif faction == "Alliance" then
				return "QuestPortraitIcon-Alliance-small", true, 15, 15
			end
			
			return atlasList[faction]
		end,

		["bang"] = function() return "worldquest-questmarker-questbang", nil, 6, 15 end,

		["question"] = function() return "worldquest-questmarker-questionmark", nil, 10, 15 end,
	}

	local function ApplyTextureToPOI(texture, width, height)
		texture:SetTexCoord(0, 1, 0, 1)
		texture:ClearAllPoints()
		texture:SetPoint("CENTER", texture:GetParent())
		texture:SetSize(width or 32, height or 32)
	end

	local function ApplyAtlasTexturesToPOI(button, normal, pushed, highlight, width, height)
		button:SetSize(20, 20)
		button:SetNormalAtlas(normal)
		ApplyTextureToPOI(button:GetNormalTexture(), width, height)

		button:SetPushedAtlas(pushed)
		ApplyTextureToPOI(button:GetPushedTexture(), width, height)

		button:SetHighlightAtlas(highlight)
		ApplyTextureToPOI(button:GetHighlightTexture(), width, height)
	end

	local function ApplyStandardTexturesToPOI(button, selected)
		button:SetSize(20, 20)
		button:SetNormalTexture("Interface/WorldMap/UI-QuestPoi-NumberIcons")
		ApplyTextureToPOI(button:GetNormalTexture())
		if selected then
			button:GetNormalTexture():SetTexCoord(0.500, 0.625, 0.375, 0.5)
		else
			button:GetNormalTexture():SetTexCoord(0.875, 1, 0.375, 0.5)
		end

		button:SetPushedTexture("Interface/WorldMap/UI-QuestPoi-NumberIcons")
		ApplyTextureToPOI(button:GetPushedTexture())
		if selected then
			button:GetPushedTexture():SetTexCoord(0.375, 0.500, 0.375, 0.5)
		else
			button:GetPushedTexture():SetTexCoord(0.750, 0.875, 0.375, 0.5)
		end

		button:SetHighlightTexture("Interface/WorldMap/UI-QuestPoi-NumberIcons")
		ApplyTextureToPOI(button:GetHighlightTexture())
		button:GetHighlightTexture():SetTexCoord(0.625, 0.750, 0.875, 1)
	end

	local function WorldMap_SetupWorldQuestButton(button, worldQuestType, rarity, isElite, tradeskillLineIndex, inProgress, selected, isCriteria, isSpellTarget, isEffectivelyTracked)
		if rarity == LE_WORLD_QUEST_QUALITY_COMMON then
			ApplyStandardTexturesToPOI(button, selected)
		elseif rarity == LE_WORLD_QUEST_QUALITY_RARE then
			ApplyAtlasTexturesToPOI(button, "worldquest-questmarker-rare", "worldquest-questmarker-rare-down", "worldquest-questmarker-rare", 18, 18)
		elseif rarity == LE_WORLD_QUEST_QUALITY_EPIC then
			ApplyAtlasTexturesToPOI(button, "worldquest-questmarker-epic", "worldquest-questmarker-epic-down", "worldquest-questmarker-epic", 18, 18)
		end

		local tradeskillLineID = tradeskillLineIndex and select(7, GetProfessionInfo(tradeskillLineIndex))

		local texture, blah, w, h = atlasList["bang"]()


		if ( inProgress ) then
			texture, blah, w, h = atlasList["question"]()
			button.Texture:SetAtlas(texture)
			button.Texture:SetSize(w, h)
		elseif atlasList[worldQuestType] then
			texture, blah, w, h = atlasList[worldQuestType](tradeskillLineID)
			button.Texture:SetAtlas(texture, blah)
			button.Texture:SetSize(w or 10,h or 10)
			
			
		elseif worldQuestType == "corpse" then
			button.Texture:SetTexture("Interface\\Worldmap\\Skull_64Grey")
			button.Texture:SetSize(15,15)
		elseif worldQuestType == "userSelected" then
			button.Texture:SetTexture("Interface\\Worldmap\\UI-World-Icon")
			button.Texture:SetSize(20,20)
		elseif worldQuestType == "rare" then
			SetPortraitTexture(button.Texture, "target")
			button.Texture:SetSize(20,20)
		elseif worldQuestType == "treasure" then
			SetPortraitToTexture(button.Texture, "Interface\\ICONS\\Icon_TreasureMap")
			button.Texture:SetSize(20,20)
		else
			button.Texture:SetAtlas(texture, blah)
			button.Texture:SetSize(w, h)
		end
		button.Texture:SetDrawLayer("ARTWORK", 1)
		button.ring:SetDrawLayer("ARTWORK", 2)
	end

	function Quest_Compass:HideAll()
		self.line:Hide()
		self.distText:SetText("")
		self.arrow:Hide()
		self.pin:Hide()
	end

	function Quest_Compass:ShowAll()
		self.compassManager:Show()
		self.line:Show()
		self.arrow:Show()
		self.pin:Show()
	end

	local canCreateGroup = {}


	function Quest_Compass:UpdateWayPoint(px, py, wx, wy, pID, wID)
		if (self.resetNow == true) then
			self.waypointX, self.waypointY = nil, nil
			self.resetNow = nil
			return
		end

		if (wx == 0) and (wy == 0 ) then
			return self:HideAll()
		 end


		TrackTreasure(px, py, pID)

		local vID = self:GetViewedMapID()
		local vwID = HBD.GetWorldMapIDFromMapID[vID]

		if (HBD:AreMapsOnSameWorld(pID, wID) == false) or
			(HBD:AreMapsOnSameWorld(pID, vwID) == false) then
			return self:HideAll()
		end


		local _pX, _pY = HBD:TranslateZoneCoordinates(px, py, pID, vwID, true)
		local _wX, _wY = HBD:TranslateZoneCoordinates(wx, wy, wID, vwID, true)

		local viewed_pX, viewed_pY = HBD:TranslateZoneCoordinates(_pX, _pY, vwID, vID, true)
		local viewed_wX, viewed_wY = HBD:TranslateZoneCoordinates(_wX, _wY, vwID, vID, true)

		local angle, distance
		if viewed_pX and viewed_wX then
			distance = math.sqrt(math.pow((_pX - _wX), 2) + math.pow((_pY - _wY), 2))
			distance = math.floor(distance *1000000)/100
		else
			self.line:Hide()
			self.arrow:Hide()
			self.distText:SetText("")
		end

		if viewed_wX and viewed_wY and (distancce ~= 0) then
			self:ShowAll()

			local isInArea, isOnMap, numObjectives = false
			if self.questID then
				isInArea, isOnMap, numObjectives = GetTaskInfo(self.questID)
			end

			if (isInArea == true) or (distance <= (self.sets.distance or 1)) then
				self.line:Hide()
				self.arrow:Hide()
				self.distText:SetText("")
			else
				self.distText:SetText(distance)
				self.arrow:Show()

				local xDist = viewed_wX - viewed_pX
				local yDist = (viewed_wY - viewed_pY)

				local dist = math.sqrt(math.pow((xDist), 2) + math.pow((yDist), 2))

				local angle = 90 - acos(xDist/dist)

				if angle < 0 then
					angle = 360 + angle
				end
				if viewed_wY > viewed_pY then
					angle = 270 - math.abs(270 - angle)
					if viewed_wX > viewed_pX then
						angle = 180 - angle
					end
				end

				local canvas = self:GetCanvas()

				self.playerBox:SetPoint("TopLeft", WorldMapFrame)
				self.playerBox:SetPoint("BottomRight", canvas, viewed_pX, viewed_pY)
				self.wayBox:SetPoint("TopLeft", WorldMapFrame)
				self.wayBox:SetPoint("BottomRight", canvas, viewed_wX, viewed_wY)

				local wayW, wayH = self.wayBox:GetSize()
				local playW, playH = self.playerBox:GetSize()

				if math.sqrt(math.pow((playW - wayW), 2) + math.pow((playH - wayH), 2)) < .03 then
					self.line:Hide()
				else
					self.line:Show()
				end

				angle = angle * math.pi /180
				local playerAngle = GetPlayerFacing()
				local arrowAngle = 0 - (playerAngle + angle)
				
				
				
				local arrowAngle = Questra:GetDirectionToQuest(self.questID)
				self.arrow:SetRotation(arrowAngle)

				if self.sets.line == true then
					local W, H = (canvas:GetWidth()-5), ((canvas:GetHeight()))
					local level = canvas:GetFrameLevel()

					self.lineContainer:SetFrameStrata("HIGH")
					self.lineContainer:SetFrameLevel(level + 50)
					self.lineContainer:SetParent(canvas)
					self.lineContainer:SetAllPoints(canvas)

					local scale = canvas:GetScale()

					self.line:SetThickness(3/scale)

					local dist = 10 / scale

					local pAngle = (math.deg(-angle) + 90) * (math.pi/180) --in degrees
					local pX = (((W ) * (viewed_pX)) + 2.5) + math.cos(pAngle) * dist
					local pY = (H-((H) * (viewed_pY))+1.25) + math.sin(pAngle )* dist

					local _wX = ((viewed_wX)) * (W)
					local _wY = H -((viewed_wY) * (H))

					local wAngle = (math.deg(0-angle) - 90) * (math.pi/180) --in degrees
					local wX = (_wX) + math.cos(wAngle) * dist
					local wY = _wY + math.sin(wAngle )* dist

					self.line:SetStartPoint("BottomLeft", canvas, pX, pY)
					
					if self.waypointType == "userSelected" then
						self.line:SetEndPoint("BottomLeft", canvas, _wX, _wY)
					else
						self.line:SetEndPoint("BottomLeft", canvas, wX, wY)
					end
				end


			end
			
				--set pin texture according to quest type
				self.pin:Show()
				if self.waypointType == "quest" then
					self.pin.questID = self.questID
					local tagID, tagName, worldQuestType, rarity, isElite, tradeskillLineIndex, displayTimeLeft = GetQuestTagInfo(self.questID)

					local _, factionID = C_TaskQuest.GetQuestInfoByQuestID(self.questID)
					if factionID then
						local factionTitle = string.upper(GetFactionInfoByID(factionID)) -- Gets details for a specific faction by factionID.
						if factionTitle == "ALLIANCE" then
							worldQuestType = "FACTION_ALLIANCE"
						elseif factionTitle == "HORDE" then
							worldQuestType = "FACTION_HORDE"
						end
					end
					local questLogIndex = GetQuestLogIndexByID(self.questID)
					local inProgress = questLogIndex ~= 0
					WorldMap_SetupWorldQuestButton(self.pin, worldQuestType, rarity or 1, isElite, tradeskillLineIndex, inProgress, isSuperTracked, nil, nil, isTrackedWorldQuest)
				
				
					if not canCreateGroup[self.questID] then
						canCreateGroup[self.questID] = C_LFGList.CanCreateQuestGroup(self.questID)
					end

					self:ShowGroupButton(self.questID, canCreateGroup[self.questID])
					self:ShowItemButton(self.questID)
				elseif self.waypointType == "rare" then
					WorldMap_SetupWorldQuestButton(self.pin, self.waypointType, 1)
				elseif self.waypointType == "treasure" then
					WorldMap_SetupWorldQuestButton(self.pin, self.waypointType, 1)
				else
					WorldMap_SetupWorldQuestButton(self.pin, self.waypointType, 1)
				end
			
		else
			self:HideAll()
		end
	end

	function Quest_Compass:ShowGroupButton(questID, state)
		if not self.groupButton then
			self.groupButton = CreateFrame("BUTTON", nil, self.coords, "QuestObjectiveFindGroupButtonTemplate", function() end);

			self.groupButton:ClearAllPoints()
			
			self.groupButton:SetPoint("Top", self.compassClick)
			self.groupButton:SetPoint("Bottom", self.compassClick)
			self.groupButton:SetPoint("Right", self.compassClick, "Center")
			
			self.groupButton:SetPoint("Left", self.compass, "Left", 0, 0)
			
			local w, h = self.pin:GetSize()
			
			self.groupButton:SetSize(w/2,h)
			self.groupButton.Icon:SetTexture("")
			self.groupButton:SetNormalTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\lfg-open")
			self.groupButton:SetPushedTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\lfg-closed")
			self.groupButton:SetHighlightTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\lfg-highlight")
		end

		self.groupButton.questID = questID
		if state == true then
			if not InCombatLockdown() and not self.groupButton:IsShown() then
				self.groupButton:Show()
			end
		else
			if not InCombatLockdown() and self.groupButton:IsShown() then
				self.groupButton:Hide()
			end
		end
	end
	
	function Quest_Compass:ShowItemButton(questID)
		if not self.itemButton then
			self.itemButton = CreateFrame("Button", nil, self.coords, "Quest_CompassQuestObjectiveItemButtonTemplate");

			self.itemButton:ClearAllPoints()
			
			self.itemButton:SetPoint("Top", self.compassClick)
			self.itemButton:SetPoint("Bottom", self.compassClick)
			self.itemButton:SetPoint("Left", self.compassClick, "Center")
			
			self.itemButton:SetPoint("Right", self.compassClick, "Right", 0, 0)
			
			
			self.itemButton:SetNormalTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\item")
			self.itemButton:SetPushedTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\item-pushed")
			self.itemButton:SetHighlightTexture("Interface\\\AddOns\\Quest_Compass\\artwork\\lfg-highlight")
			self.itemButton:GetHighlightTexture():SetTexCoord(1,0,0,1)
			
			self.itemButton.icon:ClearAllPoints()
			self.itemButton.icon:SetPoint("Center", 10, 0)
			self.itemButton.icon:SetSize(20, 20)
			self.itemButton.icon:SetDrawLayer("ARTWORK")
			self.itemButton:GetNormalTexture():SetDrawLayer("BACKGROUND")
			self.itemButton:GetPushedTexture():SetDrawLayer("BACKGROUND")
			self.itemButton:Hide()

			self.itemButton:EnableMouse(true)
			
			self.itemButton:SetScript("OnEnter", function(self)
					GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
					
					GameTooltip:SetItemByID(self:GetID())
					
			end)
			
		end

		

			


		if GetQuestLogIndexByID(questID) and GetQuestLogSpecialItemInfo(GetQuestLogIndexByID(questID)) then
						
			local questLogIndex = GetQuestLogIndexByID(questID)
			local link, item, charges, showItemWhenComplete = GetQuestLogSpecialItemInfo(questLogIndex)
			
			if self.itemButton.lastItem ~= item then
				
				local itemID = GetItemInfoInstant(link)
				self.itemButton.charges = charges;
				self.itemButton.rangeTimer = -1;
				SetItemButtonTexture(self.itemButton, item);
				SetItemButtonCount(self.itemButton, charges);
				QuestObjectiveItem_UpdateCooldown(self.itemButton);
					
				if not InCombatLockdown() then
					self.itemButton:SetAttribute('type', 'macro')
					self.itemButton:SetAttribute('macrotext', '/use ' .. GetItemInfo(itemID))
					self.itemButton:SetID(itemID)
				end
				SetPortraitToTexture(self.itemButton.icon, self.itemButton.icon:GetTexture())
				self.itemButton.icon:SetTexCoord(.1, .9, .1, .9)
				self.itemButton.lastItem = item
			end
			if not InCombatLockdown() and not self.itemButton:IsShown() then
				self.itemButton:Show()
			end
		else
			if not InCombatLockdown() and self.itemButton:IsShown() then
				self.itemButton:Hide()
			end
		end
	end
	
	--[[
	local index = GetQuestLogIndexByID(questID) 
	
	GetQuestLogSpecialItemCooldown(index)
GetQuestLogSpecialItemInfo(index)
IsQuestLogSpecialItemInRange(index [,target])
UseQuestLogSpecialItem(index)
	--]]
	local sessionIgnore = {}
	Quest_Compass.sessionIgnore = sessionIgnore
	
	function Quest_Compass:IgnoreQuest(btn)
		if IsControlKeyDown() then
			--clear ignored
			if btn == "RightButton" then
				self.sets.ignored = {}
			else
				self.sessionIgnore = {}
			end
			return
		end

		if self.waypointType == "treasure" then

				local  message = [[I found some treasure at: ]]..format("%.2f, %.2f", treasureX*100 or 0, treasureY*100 or 0)
				
				
				CTL:SendChatMessage("NORMAL", "Quest_Compass", message, "CHANNEL", "COMMON", 1)

			treasureShared = true
		elseif self.waypointType == "rare" then
			if btn == "RightButton" then
				--never offer to share this rare again.
			else
				--Share to General chat
				
				local inf = rareTargetInfo
				
				local  message = [[Rare Enemy Located: ]]..[[<]]..inf.name..[[>]]..[[ Health: ]]..inf.health..[[% Located at: ]].. format("%.2f, %.2f", inf.x*100 or 0, inf.y*100 or 0)
				
				
				CTL:SendChatMessage("NORMAL", "Quest_Compass", message, "CHANNEL", "COMMON", 1)
			end
			
			
			
		elseif self.waypointType == "userSelected" then
			if btn == "RightButton" then
				self.resetNow = true
			else
				--open map
				WorldMapFrame:Show()
				WorldMapFrame:SetMapID(self.waypointID)
			end
		elseif self.waypointType == "corpse" then
			if btn == "RightButton" then
				self.ignoreDeath = true
			else
				WorldMapFrame:Show()
				WorldMapFrame:SetMapID(self.waypointID)
			end
		elseif self.waypointType == "quest" then
			local questID = self.questID

			if IsAltKeyDown() then
				if btn == "RightButton" then
					--ignore always
					self.sets.ignored = self.sets.ignored or {}
					self.sets.ignored[questID] = true
				else
					--ignore this session
					sessionIgnore[questID] = true
				end
				
				QuestObjectiveTracker_UntrackQuest(nil, questID)
				BonusObjectiveTracker_UntrackWorldQuest(questID)

			elseif btn == "RightButton" then
				local link = GetQuestLink(questID)
				if link then
					ChatEdit_InsertLink(link)
				end
			else
				--open map
				SetSuperTrackedQuestID(questID)
				if IsWorldQuestWatched(questID) ~= true then
					BonusObjectiveTracker_TrackWorldQuest(questID, true)
				end
				QuestMapFrame_OpenToQuestDetails(questID)
			end
		end
	end
end

--[[Getter Functions--]]
do
	function Quest_Compass:GetSets(set, value)
		if set then
			if not Quest_CompassDB[set] then
				Quest_CompassDB[set] = value
			end
			return Quest_CompassDB[set]
		end
		return Quest_CompassDB
	end

	function Quest_Compass:GetMap()
		return _G["WorldMapFrame"]
	end

	function Quest_Compass:GetViewedMapID()
		if WorldMapFrame:IsVisible() then
			return _G["WorldMapFrame"]:GetMapID()
		else
			return self:GetPlayerMapID()
		end
	end

	function Quest_Compass:GetPlayerMapID()
		return C_Map.GetBestMapForUnit("player")
	end

	function Quest_Compass:GetCanvas()
		return _G["WorldMapFrame"]:GetCanvas()
	end

	local lastIgnored

	function Quest_Compass:IsQuestIgnored(questID)
		if (self.sets.ignored and self.sets.ignored[questID]) or self.sessionIgnore[questID] then
			return true
		end
	end

	local blankTask = {}

	local worldQuestTypes = {
		[LE_QUEST_TAG_TYPE_PET_BATTLE] = "Battle Pet",
		[LE_QUEST_TAG_TYPE_DUNGEON] = "Dungeon",
		[LE_QUEST_TAG_TYPE_PVP] = "PvP",
		[LE_QUEST_TAG_TYPE_PROFESSION] = "Profession",
		[LE_QUEST_TAG_TYPE_RAID] = "Raid",
		[LE_QUEST_TAG_TYPE_INVASION] = "Invasion",
	}

	local function IsWorldQuestTypeIgnored(questType)
		return Quest_Compass.sets.ignoreQuestTypes[worldQuestTypes[questType]]
	end

	local function ShouldIgnore(questID, questType)
		if (Quest_Compass:IsQuestIgnored(questID) == true) or (IsWorldQuestTypeIgnored(questType)) then
			return true
		end
	end

	function Quest_Compass:GetQuestLocation()
		--get the id for world or standard quests
		local questID = C_QuestSession.GetSuperTrackedQuest() or GetSuperTrackedQuestID()

		local playerMapID = self:GetPlayerMapID()

		if ShouldIgnore(questID) then
			questID = nil
		end
	
		if (self.sets.nearest_worldQuests == true) and ((questID == 0) or (questID == nil)) then
			local id = Questra:GetNearestWorldQuestID(true, playerMapID, ShouldIgnore)
			
			if id then
				questID = id
			end
		end
		
		if ((questID == 0) or (questID == nil)) then
			local id = Questra:GetNearestQuestID(true, playerMapID, ShouldIgnore)
			
			if id then
				questID = id
			end
		end
				
		local x, y, mapID = Questra:GetQuestLocation(questID)
		
		local nextWay = C_QuestLog.GetNextWaypoint(questID)
		
		
		if (not mapID) or (not (questID and (x or y))) then
			return 0, 0, mapID
		end

		self.questID = questID

		return x, y, mapID
	end


	local function WorldMap_AddQuestTimeToTooltip(questID)
		local timeLeftMinutes = C_TaskQuest.GetQuestTimeLeftMinutes(questID)
		if ( timeLeftMinutes and timeLeftMinutes > 0 ) then
			local color = NORMAL_FONT_COLOR
			if ( timeLeftMinutes <= WORLD_QUESTS_TIME_CRITICAL_MINUTES ) then
				color = RED_FONT_COLOR
			end

			local timeString
			if timeLeftMinutes <= 60 then
				timeString = SecondsToTime(timeLeftMinutes * 60)
			elseif timeLeftMinutes < 24 * 60 then
				timeString = D_HOURS:format(math.floor(timeLeftMinutes) / 60)
			else
				timeString = D_DAYS:format(math.floor(timeLeftMinutes) / 1440)
			end

			GameTooltip:AddLine(BONUS_OBJECTIVE_TIME_LEFT:format(timeString), color.r, color.g, color.b)
		end
	end

	function Quest_Compass:TaskPOI_OnEnter(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")

		GameTooltip:ClearAllPoints()
		local p1, p2 = Quest_Compass.SelectProperSide(self)
		GameTooltip:SetPoint(p1, self, p2)

		if Quest_Compass.waypointType == "treasure" then
			GameTooltip:SetText("Did you find some Treasure?")
			GameTooltip:AddLine("Left Click to share in General Chat.")
			GameTooltip:Show()
			return
		elseif Quest_Compass.waypointType == "rare" then
			GameTooltip:SetText("Rare Enemy Detected")
			GameTooltip:AddLine("Left Click to share in General Chat.")
			GameTooltip:Show()
			return
		elseif Quest_Compass.waypointType == "userSelected" then
			GameTooltip:SetText("Waypoint")

			GameTooltip:AddDoubleLine(format("%.2f", Quest_Compass.waypointX*100), format("%.2f", Quest_Compass.waypointY*100))
			GameTooltip:Show()
			return
		elseif Quest_Compass.waypointType == "corpse" then
			GameTooltip:SetText(DEAD)
			GameTooltip:Show()
			return
		end

		if (not self.questID) or (self.questID and ( not HaveQuestData(self.questID) )) then
			return
		end

		local title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(self.questID) or C_QuestLog.GetQuestInfo(self.questID)

		local tagID, tagName, worldQuestType, rarity, isElite, tradeskillLineIndex, displayTimeLeft = GetQuestTagInfo(self.questID)
		local color = WORLD_QUEST_QUALITY_COLORS[rarity]

		if color then
			GameTooltip:SetText(title, color.r, color.g, color.b)
		else
			GameTooltip:SetText(title)
		end

		QuestUtils_AddQuestTypeToTooltip(GameTooltip, self.questID, NORMAL_FONT_COLOR)


		if factionID then
			local factionName = GetFactionInfoByID(factionID) -- Gets details for a specific faction by factionID.

			if ( factionName ) then
				if (capped) then
					GameTooltip:AddLine(factionName, GRAY_FONT_COLOR:GetRGB())
				else
					GameTooltip:AddLine(factionName)
				end
			end
		end
		WorldMap_AddQuestTimeToTooltip(self.questID)

		local numObjectives = C_QuestLog.GetNumQuestObjectives(self.questID) or 0

		local wayPointText = C_QuestLog.GetNextWaypointText(self.questID)

		if wayPointText then
			local color = HIGHLIGHT_FONT_COLOR
			GameTooltip:AddLine(QUEST_DASH .. wayPointText, color.r, color.g, color.b, true)
		else
			for objectiveIndex = 1, numObjectives do
				local objectiveText, objectiveType, finished = GetQuestObjectiveInfo(self.questID, objectiveIndex, false)
				if ( objectiveText and #objectiveText > 0 ) then
					local color = finished and GRAY_FONT_COLOR or HIGHLIGHT_FONT_COLOR
					GameTooltip:AddLine(QUEST_DASH .. objectiveText, color.r, color.g, color.b, true)
				end
			end
		end
		local percent = C_TaskQuest.GetQuestProgressBarInfo(self.questID)
		if ( percent ) then
			GameTooltip_ShowProgressBar(GameTooltip, 0, 100, percent, PERCENTAGE_STRING:format(percent))
		end

		GameTooltip_AddQuestRewardsToTooltip(GameTooltip, self.questID)
		GameTooltip:Show()
		GameTooltip.recalculatePadding = true
	end

	function Quest_Compass:TaskPOI_OnLeave(self)
		GameTooltip:Hide()
	end

	--saving this for later.
	local function BonusObjectiveTracker_ShowRewardsTooltip(block)
		local questID
		if ( block.id < 0 ) then
			-- this is a scenario bonus objective
			questID = C_Scenario.GetBonusStepRewardQuestID(-block.id)
			if ( questID == 0 ) then
				-- huh, no reward
				return
			end
		else
			questID = block.id
			if ( COMPLETED_BONUS_DATA[questID] ) then
				-- no tooltip for completed objectives
				return
			end
		end

		if ( HaveQuestData(questID) and GetQuestLogRewardXP(questID) == 0 and GetNumQuestLogRewardCurrencies(questID) == 0
									and GetNumQuestLogRewards(questID) == 0 and GetQuestLogRewardMoney(questID) == 0 and GetQuestLogRewardArtifactXP(questID) == 0 ) then
			GameTooltip:Hide()
			return
		end

		GameTooltip:ClearAllPoints()
		GameTooltip:SetPoint("TOPRIGHT", block, "TOPLEFT", 0, 0)
		GameTooltip:SetOwner(block, "ANCHOR_PRESERVE")

		if ( not HaveQuestData(questID) ) then
			GameTooltip:AddLine(RETRIEVING_DATA, RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b)
		else
			local isWorldQuest = block.module.ShowWorldQuests

			if (isWorldQuest) then
				local headerLine = 1
				local needsSpacer = false

				local mapID, zoneMapID = C_TaskQuest.GetQuestZoneID(questID)
				if (mapID and zoneMapID) then
					local mapInfo = C_Map.GetMapInfo(zoneMapID)

					if (mapInfo) then
						GameTooltipTextLeft1:SetFontObject(GameTooltipText)
						GameTooltip:SetText(mapInfo.name, 0.4, 0.733, 1.0)
						needsSpacer = true
						headerLine = headerLine + 1

						local title, factionID, capped = C_TaskQuest.GetQuestInfoByQuestID(questID)

						if ( factionID ) then
							local factionName = GetFactionInfoByID(factionID)
							if ( factionName ) then
								if (capped) then
									GameTooltip:AddLine(factionName, GRAY_FONT_COLOR:GetRGB())
								else
									GameTooltip:AddLine(factionName, 0.4, 0.733, 1.0)
								end
								headerLine = headerLine + 1
							end
						end
					end
				end

				QuestUtils_AddQuestTypeToTooltip(GameTooltip, questID, NORMAL_FONT_COLOR)

				if (needsSpacer) then
					GameTooltip:AddLine(" ")
					headerLine = headerLine + 1
				end

				_G["GameTooltipTextLeft"..headerLine]:SetFontObject(GameTooltipHeaderText)
				GameTooltip:AddLine(REWARDS, NORMAL_FONT_COLOR:GetRGB())
			else
				GameTooltip:SetText(REWARDS, NORMAL_FONT_COLOR:GetRGB())
			end

			GameTooltip:AddLine(isWorldQuest and WORLD_QUEST_TOOLTIP_DESCRIPTION or BONUS_OBJECTIVE_TOOLTIP_DESCRIPTION, 1, 1, 1, 1)
			GameTooltip:AddLine(" ")
			-- xp
			local xp = GetQuestLogRewardXP(questID)
			if ( xp > 0 ) then
				GameTooltip:AddLine(string.format(BONUS_OBJECTIVE_EXPERIENCE_FORMAT, xp), 1, 1, 1)
			end
			local artifactXP = GetQuestLogRewardArtifactXP(questID)
			if ( artifactXP > 0 ) then
				GameTooltip:AddLine(string.format(BONUS_OBJECTIVE_ARTIFACT_XP_FORMAT, artifactXP), 1, 1, 1)
			end
			-- currency
			QuestUtils_AddQuestCurrencyRewardsToTooltip(questID, GameTooltip)
			-- honor
			local honorAmount = GetQuestLogRewardHonor(questID)
			if ( honorAmount > 0 ) then
				GameTooltip:AddLine(BONUS_OBJECTIVE_REWARD_WITH_COUNT_FORMAT:format("Interface\\ICONS\\Achievement_LegionPVPTier4", honorAmount, HONOR), 1, 1, 1)
			end
			-- money
			local money = GetQuestLogRewardMoney(questID)
			if ( money > 0 ) then
				SetTooltipMoney(GameTooltip, money, nil)
			end
			-- items
			local numQuestRewards = GetNumQuestLogRewards(questID)
			for i = 1, numQuestRewards do
				local name, texture, numItems, quality, isUsable = GetQuestLogRewardInfo(i, questID)
				local text
				if ( numItems > 1 ) then
					text = string.format(BONUS_OBJECTIVE_REWARD_WITH_COUNT_FORMAT, texture, HIGHLIGHT_FONT_COLOR:WrapTextInColorCode(numItems), name)
				elseif( texture and name ) then
					text = string.format(BONUS_OBJECTIVE_REWARD_FORMAT, texture, name)
				end
				if( text ) then
					local color = ITEM_QUALITY_COLORS[quality]
					GameTooltip:AddLine(text, color.r, color.g, color.b)
				end
			end
		end
		GameTooltip:Show()
		block.module.tooltipBlock = block
	end

end

Quest_Compass.Rares = {
	[124399] = 124399,
	[160867] = 160867,
	[151719] = 151719,
	[158636] = 158636,
	[152548] = 152548,
	[157170] = 157170,
	[157266] = 157266,
	[160868] = 160868,
	[152007] = 152007,
	[144644] = 144644,
	[157171] = 157171,
	[151625] = 151625,
	[152677] = 152677,
	[150191] = 150191,
	[152040] = 152040,
	[135497] = 135497,
	[152359] = 152359,
	[154495] = 154495,
	[154559] = 154559,
	[151308] = 151308,
	[152360] = 152360,
	[151627] = 151627,
	[152552] = 152552,
	[152361] = 152361,
	[151883] = 151883,
	[162370] = 162370,
	[152712] = 152712,
	[152553] = 152553,
	[151852] = 151852,
	[151884] = 151884,
	[152681] = 152681,
	[149653] = 149653,
	[153000] = 153000,
	[157176] = 157176,
	[154148] = 154148,
	[160810] = 160810,
	[154467] = 154467,
	[160874] = 160874,
	[160906] = 160906,
	[155583] = 155583,
	[152555] = 152555,
	[152556] = 152556,
	[149847] = 149847,
	[153226] = 153226,
	[157593] = 157593,
	[154342] = 154342,
	[151569] = 151569,
	[157466] = 157466,
	[153928] = 153928,
	[151124] = 151124,
	[152431] = 152431,
	[153228] = 153228,
	[154153] = 154153,
	[152464] = 152464,
	[154600] = 154600,
	[157469] = 157469,
	[152465] = 152465,
	[157183] = 157183,
	[157470] = 157470,
	[152657] = 152657,
	[150394] = 150394,
	[157120] = 157120,
	[158491] = 158491,
	[153200] = 153200,
	[151702] = 151702,
	[157153] = 157153,
	[157472] = 157472,
	[154604] = 154604,
	[157473] = 157473,
	[162372] = 162372,
	[152182] = 152182,
	[162352] = 162352,
	[152756] = 152756,
	[152788] = 152788,
	[162198] = 162198,
	[160872] = 160872,
	[151609] = 151609,
	[154701] = 154701,
	[150575] = 150575,
	[155703] = 155703,
	[152757] = 152757,
	[153898] = 153898,
	[152566] = 152566,
	[157188] = 157188,
	[157160] = 157160,
	[151897] = 151897,
	[160920] = 160920,
	[155531] = 155531,
	[162147] = 162147,
	[162142] = 162142,
	[157157] = 157157,
	[154333] = 154333,
	[157476] = 157476,
	[158528] = 158528,
	[154225] = 154225,
	[162140] = 162140,
	[161033] = 161033,
	[154576] = 154576,
	[152568] = 152568,
	[160968] = 160968,
	[151623] = 151623,
	[160623] = 160623,
	[153206] = 153206,
	[162163] = 162163,
	[151995] = 151995,
	[157279] = 157279,
	[152569] = 152569,
	[160878] = 160878,
	[160876] = 160876,
	[160825] = 160825,
	[157287] = 157287,
	[156299] = 156299,
	[162196] = 162196,
	[154578] = 154578,
	[152570] = 152570,
	[160826] = 160826,
	[160631] = 160631,
	[158531] = 158531,
	[151933] = 151933,
	[158595] = 158595,
	[160922] = 160922,
	[152794] = 152794,
	[151296] = 151296,
	[160532] = 160532,
	[151870] = 151870,
	[159087] = 159087,
	[154739] = 154739,
	[158633] = 158633,
	[150468] = 150468,
	[152795] = 152795,
	[157162] = 157162,
	[158597] = 158597,
	[158594] = 158594,
	[158557] = 158557,
	[157290] = 157290,
	[151202] = 151202,
	[152764] = 152764,
	[157468] = 157468,
	[157443] = 157443,
	[157390] = 157390,
	[150342] = 150342,
	[152414] = 152414,
	[157291] = 157291,
	[160893] = 160893,
	[155060] = 155060,
	[152542] = 152542,
	[156654] = 156654,
	[157267] = 157267,
	[153658] = 153658,
	[152415] = 152415,
	[151934] = 151934,
	[152567] = 152567,
	[152001] = 152001,
	[151948] = 151948,
	[156655] = 156655,
	[150583] = 150583,
	[157164] = 157164,
	[152416] = 152416,
	[152448] = 152448,
	[162170] = 162170,
	[152397] = 152397,
	[157134] = 157134,
	[154106] = 154106,
	[157146] = 157146,
	[156078] = 156078,
	[153205] = 153205,
	[151684] = 151684,
	[162171] = 162171,
	[156083] = 156083,
	[152290] = 152290,
	[157167] = 157167,
	[154394] = 154394,
	[150937] = 150937,
	[154968] = 154968,
	[151940] = 151940,
	[162172] = 162172,
	[152113] = 152113,
	[152291] = 152291,
	[152323] = 152323,
	[154490] = 154490,
	[154447] = 154447,
	[152545] = 152545,
	[162141] = 162141,
	[162173] = 162173,
	[160930] = 160930,
	[154332] = 154332,
	[154087] = 154087,
	[152682] = 152682,
	[155958] = 155958,
	[151878] = 151878,
	[151672] = 151672,
}


function WorldMapFrame:SetOverlayFrameLocation(frame, location)
	frame:ClearAllPoints()
	frame:SetPoint("Bottom", Quest_Compass.mapCoordshandle, "Bottom", 0, -10)
	frame:SetFrameStrata("FULLSCREEN")
	frame:SetScale(.9)
end


Questra = Questra or {}

local function GetPlayerMapID()
	return C_Map.GetBestMapForUnit("player")
end

local function GetViewedMapID()
	if WorldMapFrame:IsVisible() then
		return _G["WorldMapFrame"]:GetMapID()
	else
		return GetPlayerMapID()
	end
end

function Questra:GetQuestLocation(questId)
	local playerMapID = GetViewedMapID()
	if QuestUtils_IsQuestWorldQuest(questId) == true then
	
		local mapID = C_TaskQuest.GetQuestZoneID(questId) or GetQuestUiMapID(questId) or playerMapID
		x, y = C_TaskQuest.GetQuestLocation(questId, mapID)
	
		local _x, _y = HBD:TranslateZoneCoordinates(x, y, mapID, HBD.GetWorldMapIDFromMapID[mapID], true)
		local _x, _y = HBD:TranslateZoneCoordinates(_x, _y, HBD.GetWorldMapIDFromMapID[mapID], playerMapID, true)

		return _x , _y , playerMapID or mapID
	else
		local mapID, x, y = C_QuestLog.GetNextWaypoint(questId)
		

		
		if not (x and y) then
			mapID = GetQuestUiMapID(questId) or playerMapID
			local mapQuests = C_QuestLog.GetQuestsOnMap(mapID)
			if mapQuests then
				for i, info in ipairs(mapQuests) do
					if questId == info.questID then
						local _x, _y = HBD:TranslateZoneCoordinates(info.x, info.y, mapID, HBD.GetWorldMapIDFromMapID[mapID], true)
						local _x, _y = HBD:TranslateZoneCoordinates(_x, _y, HBD.GetWorldMapIDFromMapID[mapID], playerMapID, true)
						return _x or x, _y or y, playerMapID or mapID
					end
				end
			end
		end
		return x, y, mapID
	end
end

local questMapInfo = {}
function Questra:GetQuestsForMap(uiMapID)
	wipe(questMapInfo)
	local uiMapID = uiMapID or GetViewedMapID()
	
	local mapQuests = C_QuestLog.GetQuestsOnMap(uiMapID)
	if mapQuests then
		for i, info in ipairs(mapQuests) do
			if questId == info.questId then
				info.x, info.y, info.mapID = self:GetQuestLocation(info.questId)
				tinsert(questMapInfo, info)
			end
		end
	end
	return questMapInfo
end


local worldQuestMapInfo = {}
local info = {}
function Questra:GetWorldQuestsForMap(uiMapID)
	local uiMapID = uiMapID or GetViewedMapID()

	local taskInfo
	if (uiMapID) then
		taskInfo = C_TaskQuest.GetQuestsForPlayerByMapID(uiMapID)
	end

	if taskInfo then
		for i, info in ipairs(taskInfo) do
			local questID = info.questId
			if QuestUtils_IsQuestWorldQuest(questID) and HaveQuestData(questID) then
				local tagID, tagName, worldQuestType, rarity, isElite, tradeskillLineIndex, displayTimeLeft = GetQuestTagInfo(questID)
				local info = {}
				
				info.x, info.y, info.mapID = self:GetQuestLocation(questID)
				
				info.questID = questID
				info.tagID = tagID
				info.tagName = tagName
				info.worldQuestType = worldQuestType
				info.rarity = rarity
				info.isElite = isElite
				info.tradeskillLineIndex = tradeskillLineIndex
				info.tagID = displayTimeLeft

				tinsert(worldQuestMapInfo, info)
			end
		end
	end
	return worldQuestMapInfo
end

local blankFunc = function() end

function Questra:GetNearestQuestID(uiMapID, fallBackID, shouldIgnore)
	shouldIgnore = shouldIgnore or blankFunc
	local uiMapID = uiMapID or GetViewedMapID()
	if uiMapID == true then
		uiMapID = HBD.GetWorldMapIDFromMapID[C_Map.GetBestMapForUnit("player") or GetViewedMapID()]
	end

	local quests = self:GetQuestsForMap(uiMapID)
	local distance = math.huge
	local questId
	for i, quest in pairs(quests) do
		local dist = self:GetDistanceToQuest(quest.questId)
		if dist < distance  and (shouldIgnore(quest.questId) == nil) then
			questId = quest
			distance = dist
		end
	end
		
	return questId --self:GetQuestInfo(questId)
end

function Questra:GetNearestWorldQuestID(uiMapID, fallBackID, shouldIgnore)
	shouldIgnore = shouldIgnore or blankFunc
	
	local uiMapID = (uiMapID == true) and HBD.GetWorldMapIDFromMapID[ GetViewedMapID()] or uiMapID or GetViewedMapID()
	
	local quests = uiMapID and C_TaskQuest.GetQuestsForPlayerByMapID(uiMapID) or {}

	local distance, questId = math.huge	
	for i, quest in ipairs(quests) do
		if quest.questId then
			if QuestUtils_IsQuestWorldQuest(quest.questId) and HaveQuestData(quest.questId) then
				local dist = self:GetDistanceToQuest(quest.questId)
				local _, _, worldQuestType = GetQuestTagInfo(quest.questId)
				if (dist and dist <= distance) and (shouldIgnore(quest.questId, worldQuestType) == nil) then
					distance = dist
					questId = quest.questId
				end
			end
		end
	end
	if distance ~= math.huge then
		return questId
	end
end

function Questra:GetDistanceToQuest(questId)
	local _pX, _pY, playerMapID = HBD:GetPlayerZonePosition(true)
	_pX, _pY = _pX or 0, _pY or 0
	local x, y, uiMapID = self:GetQuestLocation(questId)

	local _x, _y = HBD:TranslateZoneCoordinates(x, y, uiMapID, HBD.GetWorldMapIDFromMapID[uiMapID], true)
	local _x, _y = HBD:TranslateZoneCoordinates(_x, _y, HBD.GetWorldMapIDFromMapID[uiMapID], GetViewedMapID(), true)

	local _pX, _pY = HBD:TranslateZoneCoordinates(_pX, _pY, playerMapID, GetViewedMapID(), true)

	if (not (_x and _y)) or (not (_pX and _pY)) then
		return math.huge
	end

	return math.sqrt(math.pow((_pX - _x), 2) + math.pow((_pY - _y), 2))
end

function Questra:GetDirectionToQuest(questId)
	local x, y, uiMapID = self:GetQuestLocation(questId)
	local _pX, _pY, playerMapID = HBD:GetPlayerZonePosition(true)

	local _x, _y = HBD:TranslateZoneCoordinates(x, y, uiMapID, HBD.GetWorldMapIDFromMapID[uiMapID], true)
	local x, y = HBD:TranslateZoneCoordinates(_x, _y, HBD.GetWorldMapIDFromMapID[uiMapID], playerMapID, true)
	
	
	local __pX, _pY = HBD:TranslateZoneCoordinates(_pX, _pY, playerMapID, HBD.GetWorldMapIDFromMapID[playerMapID], true)
	local __pX, _pY = HBD:TranslateZoneCoordinates(_pX, _pY, HBD.GetWorldMapIDFromMapID[playerMapID], playerMapID, true)

	local xDist = x - _pX
	local yDist = y - _pY

	local dist = math.sqrt(math.pow((xDist), 2) + math.pow((yDist), 2))

	local angle = 90 - acos(xDist/dist)

	if angle < 0 then
		angle = 360 + angle
	end
	if y > _pY then
		angle = 270 - math.abs(270 - angle)
		if x > _pX then
			angle = 180 - angle
		end
	end

	angle = angle * math.pi /180
	local playerAngle = GetPlayerFacing()
	local arrowAngle = (playerAngle + angle)
	
	return -arrowAngle
end

function Questra:GetQuestInfo(questId)
	local questInfo = {}
	
	questInfo.x, questInfo.y, questInfo.mapID = self:GetQuestLocation(questId)
	questInfo.title, questInfo.factionID, questInfo.capped = C_TaskQuest.GetQuestInfoByQuestID(questId) or C_QuestLog.GetQuestInfo(questId)
	questInfo.tagID, questInfo.tagName, questInfo.worldQuestType, questInfo.rarity, questInfo.isElite, questInfo.tradeskillLineIndex, questInfo.displayTimeLeft = GetQuestTagInfo(questId)
	questInfo.distance = self:GetDistanceToQuest(questId)
	
	return questInfo
end